"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertState = exports.createMergedState = exports.mergeStateMapWithPart = void 0;
const _utils_1 = require("./@utils");
function mergeStateMapWithPart(path, stateMap, statePart) {
    const states = path
        .map((_key, index) => stateMap.get(index))
        .reverse();
    const pendingStatePartMap = new Map(Object.entries(statePart));
    for (const state of states) {
        for (const [statePartKey, value] of pendingStatePartMap) {
            if (statePartKey in state) {
                state[statePartKey] = value;
                pendingStatePartMap.delete(statePartKey);
            }
        }
    }
    if (pendingStatePartMap.size > 0) {
        throw new Error(`Failed to find value of ${Array.from(pendingStatePartMap.keys(), key => JSON.stringify(key)).join('/')} to update`);
    }
}
exports.mergeStateMapWithPart = mergeStateMapWithPart;
function createMergedState(stateMap) {
    const states = Array.from(stateMap.values()).reverse();
    return (0, _utils_1.createMergedObjectProxy)(states);
}
exports.createMergedState = createMergedState;
function assertState(state, key) {
    if (state === undefined) {
        throw new Error(`State ${JSON.stringify(key)} is missing and no default value is provided`);
    }
}
exports.assertState = assertState;
//# sourceMappingURL=@state.js.map