"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RouteEntry = void 0;
const tslib_1 = require("tslib");
const mobx_1 = require("mobx");
const _state_1 = require("../@state");
const _utils_1 = require("../@utils");
class RouteEntry {
    constructor(router, path, stateMap, previous, pendingStatePart) {
        Object.defineProperty(this, "router", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: router
        });
        Object.defineProperty(this, "path", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: path
        });
        Object.defineProperty(this, "stateMap", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: stateMap
        });
        Object.defineProperty(this, "previous", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: previous
        });
        Object.defineProperty(this, "enteringSet", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: mobx_1.observable.set()
        });
        Object.defineProperty(this, "leavingSet", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: mobx_1.observable.set()
        });
        Object.defineProperty(this, "states", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "pendingStatePart", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "mergedState", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.states = Array.from(path.keys(), index => stateMap.get(index))
            .filter((state) => state !== undefined)
            .reverse();
        this.pendingStatePart = pendingStatePart;
        this.mergedState = (0, _utils_1.createMergedObjectProxy)(() => {
            const { states, pendingStatePart } = this;
            return pendingStatePart ? [pendingStatePart, ...states] : states;
        });
    }
    get target() {
        var _a;
        return {
            path: this.path,
            stateMap: this.stateMap,
            previous: this.previous,
            statePart: (_a = this.pendingStatePart) !== null && _a !== void 0 ? _a : {},
        };
    }
    get blockedByEntering() {
        return this.enteringSet.size > 0;
    }
    get blockedByLeaving() {
        return this.leavingSet.size > 0;
    }
    get active() {
        var _a;
        return this === ((_a = this.router._active) === null || _a === void 0 ? void 0 : _a.entry);
    }
    get transition() {
        var _a;
        return this === ((_a = this.router._transition) === null || _a === void 0 ? void 0 : _a.entry);
    }
    get switching() {
        var _a;
        return this === ((_a = this.router._switching) === null || _a === void 0 ? void 0 : _a.entry);
    }
    get leaving() {
        return this.active && this.router._transition !== undefined;
    }
    updateTransitionBlock(ref, { entering, leaving }) {
        (0, mobx_1.runInAction)(() => {
            if (entering) {
                this.enteringSet.add(ref);
            }
            else {
                this.enteringSet.delete(ref);
            }
            if (leaving) {
                this.leavingSet.add(ref);
            }
            else {
                this.leavingSet.delete(ref);
            }
        });
    }
    mergePendingStatePart() {
        const pendingStatePart = this.pendingStatePart;
        if (!pendingStatePart) {
            return;
        }
        (0, mobx_1.runInAction)(() => {
            (0, _state_1.mergeStateMapWithPart)(this.path, this.stateMap, pendingStatePart);
            this.pendingStatePart = undefined;
        });
    }
}
tslib_1.__decorate([
    mobx_1.observable
], RouteEntry.prototype, "pendingStatePart", void 0);
exports.RouteEntry = RouteEntry;
//# sourceMappingURL=route-entry.js.map