"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RouteClass = exports.RouteNodeClass = exports.createRoute = void 0;
const tslib_1 = require("tslib");
const mobx_1 = require("mobx");
const _utils_1 = require("../@utils");
const routra_1 = require("../routra");
function createRoute(router, schema, path, stateMapUpdate) {
    const Constructor = schema.$exact === false ? RouteNodeClass : RouteClass;
    return Object.setPrototypeOf((state) => {
        return new Constructor(router, schema, path, new Map([...stateMapUpdate, [path.length - 1, state]]));
    }, new Constructor(router, schema, path, stateMapUpdate));
}
exports.createRoute = createRoute;
class RouteNodeClass {
    constructor($router, schema, $path, _stateMapUpdate) {
        Object.defineProperty(this, "$router", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: $router
        });
        Object.defineProperty(this, "$path", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: $path
        });
        Object.defineProperty(this, "_stateMapUpdate", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: _stateMapUpdate
        });
        Object.defineProperty(this, "$key", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_exact", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        Object.defineProperty(this, "_exactClone", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        (0, mobx_1.makeObservable)(this);
        this.$key = $path[$path.length - 1];
        for (let [key, childSchema] of Object.entries(schema)) {
            if (key.startsWith('$')) {
                continue;
            }
            if (childSchema === true) {
                childSchema = {};
            }
            this[key] = createRoute($router, childSchema, [...$path, key], _stateMapUpdate);
        }
    }
    get $exact() {
        let clone = this._exactClone;
        if (clone) {
            return clone;
        }
        clone = Object.create(this, {
            _exact: {
                value: true,
            },
        });
        this._exactClone = clone;
        return clone;
    }
    get $matched() {
        return this.$active || this.$transition || this.$switching;
    }
    get $active() {
        return this._active !== undefined;
    }
    get $transition() {
        return this._transition !== undefined;
    }
    get $switching() {
        return this._switching !== undefined;
    }
    /** @internal */
    get _active() {
        const { $router } = this;
        const active = $router._requireActive();
        if (!this._isMatched(active.entry)) {
            return undefined;
        }
        return active;
    }
    /** @internal */
    get _transition() {
        const { $router } = this;
        const transition = $router._transition;
        if (!transition || !this._isMatched(transition.entry)) {
            return undefined;
        }
        return transition;
    }
    /** @internal */
    get _switching() {
        const { $router } = this;
        const switching = $router._switching;
        if (!switching || !this._isMatched(switching.entry)) {
            return undefined;
        }
        return switching;
    }
    $view(options) {
        return routra_1.routra.$view([this], options);
    }
    _isMatched({ path: entryPath }) {
        const { $path } = this;
        return this._exact
            ? (0, _utils_1.isArrayEqual)(entryPath, $path)
            : (0, _utils_1.isArrayStartedWith)(entryPath, $path);
    }
}
tslib_1.__decorate([
    mobx_1.computed
], RouteNodeClass.prototype, "_active", null);
tslib_1.__decorate([
    mobx_1.computed
], RouteNodeClass.prototype, "_transition", null);
tslib_1.__decorate([
    mobx_1.computed
], RouteNodeClass.prototype, "_switching", null);
exports.RouteNodeClass = RouteNodeClass;
class RouteClass extends RouteNodeClass {
    constructor($router, schema, $path, stateMapUpdate) {
        super($router, schema, $path, stateMapUpdate);
    }
    get $reset() {
        return this.$router._reset(this.$path, this._stateMapUpdate);
    }
    get $push() {
        return this.$router._push(this.$path, this._stateMapUpdate);
    }
    get $replace() {
        return this.$router._replace(this.$path, this._stateMapUpdate);
    }
}
exports.RouteClass = RouteClass;
//# sourceMappingURL=route.js.map