import type { IObservableValue } from 'mobx';
import type { ChildSchemaFallback_ } from '../@schema';
import type { RouteNode__, RouteSwitching, RouteType_ } from '../route';
import { RouteEntry } from '../route';
import type { RouteKey, SchemaRecord } from '../schema';
import type { RouterBack } from './router-back';
export type Router__ = RouterClass<any>;
export interface RouterOptions<TSwitchingState extends object> {
    defaultSwitchingState?: TSwitchingState;
}
export declare class RouterClass<TSwitchingState extends object> {
    constructor(
    /** @internal */
    _schemas: SchemaRecord, 
    /** @internal */
    _options: RouterOptions<TSwitchingState>);
    get $path(): string[] | undefined;
    get $state(): object | undefined;
    get $snapshot(): Snapshot | undefined;
    get $back(): RouterBack<TSwitchingState>;
    get $ableToBack(): boolean;
    $backTo(route: RouteNode__ | RouteNode__[]): RouterBack<TSwitchingState> | undefined;
    $restore({ operation, entry, states }: Snapshot): void;
    private _startTransition;
    private _finishTransition;
    private _buildStateMap;
}
export type RouterOperation = 'reset' | 'push' | 'replace' | 'back';
export interface ActiveEntry {
    operation: RouterOperation;
    entry: RouteEntry;
}
export interface TransitionEntry {
    operation: RouterOperation;
    entry: RouteEntry;
}
export interface SwitchingEntry {
    operation: RouterOperation;
    entry: RouteEntry;
    switchingStateObservable: IObservableValue<object>;
    ref: object;
}
export type MatchEntry = ActiveEntry | TransitionEntry | SwitchingEntry;
export interface RouterSetResult {
    $completed: Promise<void>;
}
export type Router<TSchemaRecord extends SchemaRecord, TSwitchingState extends object> = RouterClass<TSwitchingState> & {
    [TKey in Extract<keyof TSchemaRecord, RouteKey>]: RouteType_<ChildSchemaFallback_<TSchemaRecord[TKey]>, TSwitchingState, {}, [
        TKey
    ]>;
};
export type RouterSwitchingState_<TRouter> = TRouter extends RouterClass<infer TSwitchingState> ? TSwitchingState : never;
export type RouterSwitchingState<TRouter extends Router__> = RouterSwitchingState_<TRouter>;
export type RouterSwitching_<TRouter> = RouteSwitching<RouterSwitchingState_<TRouter>>;
export type RouterSwitching<TRouter extends Router__> = RouterSwitching_<TRouter>;
export interface SnapshotEntry {
    path: string[];
    inputs: unknown[];
    /**
     * Indexed by path segment index; and the value is the index of the state in
     * snapshot. Using a separate states array to preserve referencing same
     * objects by different entries.
     */
    states: number[];
    previous?: SnapshotEntry;
}
export interface Snapshot {
    operation: RouterOperation;
    entry: SnapshotEntry;
    states: object[];
}
