"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ParallelViewEntry__ = exports.ParallelView__ = void 0;
const tslib_1 = require("tslib");
const mobx_1 = require("mobx");
const view_1 = require("./view");
const view_entry_1 = require("./view-entry");
class ParallelView__ extends view_1.AbstractView {
    constructor() {
        super(...arguments);
        Object.defineProperty(this, "_routeEntryToViewEntryMap", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
    }
    get _entries() {
        const matches = this._matches;
        const routeEntryToViewEntryMap = this._routeEntryToViewEntryMap;
        const pendingIteratingRouteEntries = new Set(routeEntryToViewEntryMap.keys());
        for (const match of matches) {
            const { entry } = match;
            pendingIteratingRouteEntries.delete(entry);
            if (routeEntryToViewEntryMap.has(entry)) {
                continue;
            }
            routeEntryToViewEntryMap.set(entry, new ParallelViewEntry__(match));
        }
        for (const routeEntry of pendingIteratingRouteEntries) {
            const viewEntry = routeEntryToViewEntryMap.get(routeEntry);
            viewEntry._dispose();
            routeEntryToViewEntryMap.delete(routeEntry);
        }
        return Array.from(routeEntryToViewEntryMap.values());
    }
}
tslib_1.__decorate([
    mobx_1.computed
], ParallelView__.prototype, "_entries", null);
exports.ParallelView__ = ParallelView__;
class ParallelViewEntry__ extends view_entry_1.AbstractViewEntry {
    constructor(match) {
        super();
        /** @internal */
        Object.defineProperty(this, "_match", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "$operation", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this._match = match;
        this.$operation = match.operation;
    }
    get _entering() {
        return this._match.entry.transition;
    }
    get _leaving() {
        return this._match.entry.leaving;
    }
    /** @internal */
    _dispose() {
        super._dispose();
        (0, mobx_1.runInAction)(() => {
            this._match.entry.updateTransitionBlock(this, {
                entering: false,
                leaving: false,
            });
        });
    }
    _autorunUpdateTransitionBlock() {
        const enteringEnabled = this._enteringEnabled;
        const leavingEnabled = this._leavingEnabled;
        const entered = this._entered;
        const left = this._left;
        (0, mobx_1.runInAction)(() => {
            this._match.entry.updateTransitionBlock(this, {
                entering: enteringEnabled && !entered,
                leaving: leavingEnabled && !left,
            });
        });
    }
}
exports.ParallelViewEntry__ = ParallelViewEntry__;
//# sourceMappingURL=parallel-view.js.map