"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractView = void 0;
const tslib_1 = require("tslib");
const mobx_1 = require("mobx");
const _utils_1 = require("../@utils");
class View {
    constructor($routes) {
        Object.defineProperty(this, "$routes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: $routes
        });
        Object.defineProperty(this, "_mergedEntryMap", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new WeakMap()
        });
        (0, mobx_1.makeObservable)(this);
    }
    get $entries() {
        const mergedEntryMap = this._mergedEntryMap;
        return this._entries.map(entry => {
            let mergedEntry = mergedEntryMap.get(entry);
            if (mergedEntry) {
                return mergedEntry;
            }
            mergedEntry = (0, _utils_1.createMergedObjectProxy)([
                entry,
                entry._match.entry.mergedState,
            ]);
            mergedEntryMap.set(entry, mergedEntry);
            return mergedEntry;
        });
    }
    /** @internal */
    get _matches() {
        return this.$routes
            .flatMap((route) => {
            const active = route._active;
            const transition = route._transition;
            const switching = route._switching;
            return [
                active && { route, ...active },
                transition && { route, ...transition },
                switching && { route, ...switching },
            ];
        })
            .filter((match) => match !== undefined);
    }
}
tslib_1.__decorate([
    mobx_1.computed
], View.prototype, "_matches", null);
exports.AbstractView = View;
//# sourceMappingURL=view.js.map