import { __decorate } from "tslib";
import { computed, makeObservable } from 'mobx';
import { isArrayEqual, isArrayStartedWith } from '../@utils';
import { routra } from '../routra';
export function createRoute(router, schema, path, stateMapUpdate) {
    const Constructor = schema.$exact === false ? RouteNodeClass : RouteClass;
    return Object.setPrototypeOf((state) => {
        return new Constructor(router, schema, path, new Map([...stateMapUpdate, [path.length - 1, state]]));
    }, new Constructor(router, schema, path, stateMapUpdate));
}
export class RouteNodeClass {
    constructor($router, schema, $path, _stateMapUpdate) {
        Object.defineProperty(this, "$router", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: $router
        });
        Object.defineProperty(this, "$path", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: $path
        });
        Object.defineProperty(this, "_stateMapUpdate", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: _stateMapUpdate
        });
        Object.defineProperty(this, "$key", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_exact", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        Object.defineProperty(this, "_exactClone", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        makeObservable(this);
        this.$key = $path[$path.length - 1];
        for (let [key, childSchema] of Object.entries(schema)) {
            if (key.startsWith('$')) {
                continue;
            }
            if (childSchema === true) {
                childSchema = {};
            }
            this[key] = createRoute($router, childSchema, [...$path, key], _stateMapUpdate);
        }
    }
    get $exact() {
        let clone = this._exactClone;
        if (clone) {
            return clone;
        }
        clone = Object.create(this, {
            _exact: {
                value: true,
            },
        });
        this._exactClone = clone;
        return clone;
    }
    get $matched() {
        return this.$active || this.$transition || this.$switching;
    }
    get $active() {
        return this._active !== undefined;
    }
    get $transition() {
        return this._transition !== undefined;
    }
    get $switching() {
        return this._switching !== undefined;
    }
    /** @internal */
    get _active() {
        const { $router } = this;
        const active = $router._requireActive();
        if (!this._isMatched(active.entry)) {
            return undefined;
        }
        return active;
    }
    /** @internal */
    get _transition() {
        const { $router } = this;
        const transition = $router._transition;
        if (!transition || !this._isMatched(transition.entry)) {
            return undefined;
        }
        return transition;
    }
    /** @internal */
    get _switching() {
        const { $router } = this;
        const switching = $router._switching;
        if (!switching || !this._isMatched(switching.entry)) {
            return undefined;
        }
        return switching;
    }
    $view(options) {
        return routra.$view([this], options);
    }
    _isMatched({ path: entryPath }) {
        const { $path } = this;
        return this._exact
            ? isArrayEqual(entryPath, $path)
            : isArrayStartedWith(entryPath, $path);
    }
}
__decorate([
    computed
], RouteNodeClass.prototype, "_active", null);
__decorate([
    computed
], RouteNodeClass.prototype, "_transition", null);
__decorate([
    computed
], RouteNodeClass.prototype, "_switching", null);
export class RouteClass extends RouteNodeClass {
    constructor($router, schema, $path, stateMapUpdate) {
        super($router, schema, $path, stateMapUpdate);
    }
    get $reset() {
        return this.$router._reset(this.$path, this._stateMapUpdate);
    }
    get $push() {
        return this.$router._push(this.$path, this._stateMapUpdate);
    }
    get $replace() {
        return this.$router._replace(this.$path, this._stateMapUpdate);
    }
}
//# sourceMappingURL=route.js.map