import { __decorate } from "tslib";
import { autorun, computed, makeObservable, observable, runInAction } from 'mobx';
import { createMergedObjectProxy } from '../@utils';
class ViewEntryClass {
    constructor() {
        Object.defineProperty(this, "$key", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: getNextViewEntryKey()
        });
        Object.defineProperty(this, "_enteringEnabled", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        Object.defineProperty(this, "_leavingEnabled", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        Object.defineProperty(this, "_entered", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        Object.defineProperty(this, "_left", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        Object.defineProperty(this, "_autorunDisposer", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "enteringTransitionActivity", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "leavingTransitionActivity", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "$transition", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: ({ entering, leaving, }) => {
                runInAction(() => {
                    if (typeof entering === 'boolean') {
                        this._enteringEnabled = entering;
                    }
                    if (typeof leaving === 'boolean') {
                        this._leavingEnabled = leaving;
                    }
                });
            }
        });
        makeObservable(this);
        this._autorunDisposer = autorun(() => this._autorunUpdateTransitionBlock());
    }
    get $entering() {
        if (!this._entering) {
            return undefined;
        }
        let activity = this.enteringTransitionActivity;
        if (!activity) {
            const operation = this.$operation;
            activity = Object.freeze({
                $operation: operation,
                $complete: () => {
                    if (!this._enteringEnabled) {
                        return false;
                    }
                    if (!this._entered) {
                        runInAction(() => {
                            this._entered = true;
                        });
                    }
                    return true;
                },
            });
        }
        return activity;
    }
    get $leaving() {
        if (!this._leaving) {
            return undefined;
        }
        let activity = this.leavingTransitionActivity;
        if (!activity) {
            const operation = this.$match.$router._requireTransition().operation;
            activity = Object.freeze({
                $operation: operation,
                $complete: () => {
                    if (!this._leavingEnabled) {
                        return false;
                    }
                    if (!this._left) {
                        runInAction(() => {
                            this._left = true;
                        });
                    }
                    return true;
                },
            });
        }
        return activity;
    }
    get $switching() {
        const match = this._match;
        const route = match.route;
        const switchingTo = route._switching;
        if (switchingTo && match.entry === switchingTo.entry) {
            const shared = Object.freeze({
                $operation: switchingTo.operation,
                $rel: 'to',
            });
            return createMergedObjectProxy(() => [
                shared,
                switchingTo.switchingStateObservable.get(),
            ]);
        }
        const switchingFrom = route.$router._switching;
        if (route.$active && switchingFrom) {
            const shared = Object.freeze({
                $operation: switchingFrom.operation,
                $rel: 'from',
            });
            return createMergedObjectProxy(() => [
                shared,
                switchingFrom.switchingStateObservable.get(),
            ]);
        }
        return undefined;
    }
    get $match() {
        return this._match.route;
    }
    /** @internal */
    _dispose() {
        this._autorunDisposer();
    }
}
__decorate([
    observable
], ViewEntryClass.prototype, "_enteringEnabled", void 0);
__decorate([
    observable
], ViewEntryClass.prototype, "_leavingEnabled", void 0);
__decorate([
    observable
], ViewEntryClass.prototype, "_entered", void 0);
__decorate([
    observable
], ViewEntryClass.prototype, "_left", void 0);
__decorate([
    computed
], ViewEntryClass.prototype, "$switching", null);
export const AbstractViewEntry = ViewEntryClass;
let lastViewEntryKey = 0;
function getNextViewEntryKey() {
    return ++lastViewEntryKey;
}
//# sourceMappingURL=view-entry.js.map