import { __decorate } from "tslib";
import { computed, makeObservable } from 'mobx';
import { createMergedObjectProxy } from '../@utils';
class View {
    constructor($routes) {
        Object.defineProperty(this, "$routes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: $routes
        });
        Object.defineProperty(this, "_mergedEntryMap", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new WeakMap()
        });
        makeObservable(this);
    }
    get $entries() {
        const mergedEntryMap = this._mergedEntryMap;
        return this._entries.map(entry => {
            let mergedEntry = mergedEntryMap.get(entry);
            if (mergedEntry) {
                return mergedEntry;
            }
            mergedEntry = createMergedObjectProxy([
                entry,
                entry._match.entry.mergedState,
            ]);
            mergedEntryMap.set(entry, mergedEntry);
            return mergedEntry;
        });
    }
    /** @internal */
    get _matches() {
        return this.$routes
            .flatMap((route) => {
            const active = route._active;
            const transition = route._transition;
            const switching = route._switching;
            return [
                active && { route, ...active },
                transition && { route, ...transition },
                switching && { route, ...switching },
            ];
        })
            .filter((match) => match !== undefined);
    }
}
__decorate([
    computed
], View.prototype, "_matches", null);
export const AbstractView = View;
//# sourceMappingURL=view.js.map